"""
Pyro MessageBus:  a simple pub/sub message bus.
Provides a way of communicating where the sender and receivers are fully decoupled.

Pyro - Python Remote Objects.  Copyright by Irmen de Jong (irmen@razorvine.net).
"""

PYRO_MSGBUS_NAME = "Pyro.MessageBus"
